# supply_instrument_cfr.R
# For public via CFR
# will place in zipped folder with all data except Saiz shouldn't share without authorization
# also excludes a large IPUMS file that features 1980 and 2011 3-year ACS employment by geography and industry -- this can be recollected via IPUMS interface: see the .do file in this directory
# Tom Davidoff
# reg and outreg package available on request -- just OLS with clustering
# 05/29/15

library(regoutreg)

#figure

MISSING <- -9

N <- 10
constant <- 5
grow <- 2
p <- seq(N)
qd <- constant/p
qd_grow <- constant*qd
qs_elastic <- p
qs_inelastic <- sqrt(p)
pdf("docs/supply_instrument/text/demand_supply.pdf")
plot(qs_elastic,p,type="l",lty=1,ylim=c(-1,10),xlab="quantity",ylab="price")
lines(qd_grow,p,lty=3)
lines(qs_inelastic,p,lty=2)
lines(qd,p,lty=4)
legend("bottomright",legend=c("elastic supply","inelastic supply","high demand growth","low demand growth"),lty=seq(4))
points(1,1,pch="O")
shift <- N*.03
points(shift+constant^(1/3),constant^(1/1.5),pch="A")
points(shift+constant^(1/1.5),constant^(2/1.5),pch="B")
points(shift+constant^(1/2),constant^(1/2),pch="C")
points(shift+constant^(2/2),constant^(2/2),pch="D")
dev.off()

# create a list of counties that provides cbsa (?), metrodiv (?), msanecma (saiz), and cbsa name (FMHPI) Saiz to the county level
county_geo <- read.csv("Data/county_cbsa_13.csv")
county_geo$county <- county_geo$state*1000 + county_geo$county
sand_states <- c(4,6,12,32)
county_geo$sand <- 0
for (j in sand_states) {
	county_geo$sand <- county_geo$sand + (county_geo$state==j)
}

bb <- read.csv("Data/county_msa_pmsa_cbsa_metrodiv_mable.csv",skip=1)
bb$cbsa <- bb$metrodiv <- NULL
names(bb)[length(names(bb)) ]<- "afact" # important for weight
county_geo <- merge(county_geo,bb,by="county")
county_geo$msanecma <- county_geo$pmsa
county_geo$msanecma[county_geo$pmsa==9999] <- county_geo$msacmsa[county_geo$pmsa==9999]
summary(county_geo)

# Data/nfips.txt | awk -F ' ' '{if ($3!=NULL && NR<=66 && NR>12 && $1>100 && $2>0) {{county = $2*1000 + $3} { if (county>0) printf "%d,%d\n",county,$1}}}' | sort  -k 1b,1 > $dire/county_necma from shell script
bb <- read.csv("docs/supply_instrument/data/county_necma",header=FALSE)
names(bb) <- c("county","necma")
county_geo <- merge(county_geo,bb,by="county",all.x=TRUE)
county_geo$msanecma[!is.na(county_geo$necma)] <- county_geo$necma[!is.na(county_geo$necma)]


# Saiz measures
#header: msanecma	population	msaname	WRLURI	FLAT_SHARE_50_15	S_LAND_50	lu11	lu91	lu92	unaval	elasticity
# This data must come from Saiz himself
saiz <- read.table("Data/HOUSING_SUPPLY.dat",header=T,sep="\t")
coastalnames = c("Baltimore","Boston","Bremerton","Los Angeles","New Haven","New York","Newark","New London","Oakland","Philadelphia","Portland","Salinas","San Diego","San Francisco","San Jose","San Luis Obispo","Santa Barbara","Santa Cruz","Santa Rosa","Seattle","Tacoma","Trenton","Vallejo","Ventura","Washington, DC","Wilmington-Newark") 
saiz$coastal <- 0
for (j in coastalnames) {
	saiz$coastal <- saiz$coastal + grepl(j,saiz$msaname)
}

length(levels(factor(saiz$msanecma)))
saiz <- merge(saiz,county_geo,by="msanecma")
length(levels(factor(saiz$msanecma)))

# Now housing units and other county level stuff
u_80 <- read.csv("Data/county_units_1980_nhgis.csv")
u_80$county <- 100*u_80$STATEA+u_80$COUNTYA/10
u_80$county[u_80$county==12025] <- 12086
u_80$units_80 <-  u_80$V0000001..NT6..STF3...Total 

for (n in names(u_80)) {
	if (n %in% list("county","units_80") == FALSE) {
			u_80[[n]] <- NULL
	}
}

units_12 <- read.csv("Data/county_units_12_acs3.csv",skip=1)
summary(units_12)
units_12$county <- units_12$Id2
units_12$units_12 <- units_12$Estimate..Total
for (n in names(units_12)) {
	if (n %in% list("county","units_12") == FALSE) {
			units_12[[n]] <- NULL
	}
}

saiz <- merge(saiz,u_80,by="county")
saiz <- merge(saiz,units_12,by="county")
print(length(saiz$county))


# Johnson Goldwater (ultimately not used in paper bc missing data too much)

# state icpsr codes
icpsr = read.csv("docs/supply_instrument/data/icpsrcnt.csv")
icpsr$county_fips <- icpsr$STATEFIPS*1000 + floor(icpsr$County.cod/10)
icpsr$county_name <- toupper(icpsr$County)

elect <- read.csv("docs/supply_instrument/data/icpsr_election.txt",header=FALSE) # from icpsr_election.jl
names(elect) <- c("state","county_name","johnson","total_64","mcgovern","total_72")
for (i in list("state","johnson","total_64","mcgovern","total_72")) {
	elect[[i]] <- as.numeric(substring(elect[[i]],1,8))
}
summary(elect)
for (sym in list(","," ","-",":")) {
	elect$county_name <- gsub(sym,"",elect$county_name)
	icpsr$county_name <- gsub(sym,"",icpsr$county_name)
}
elect <- merge(elect,icpsr,by.x=c("state","county_name"),by.y=c("STATEICP","county_name"))
county <- as.vector(by(elect$county_fips,elect$county_fips,mean))
johnson <- as.vector(by(elect$johnson,elect$county_fips,sum))
mcgovern <- as.vector(by(elect$mcgovern,elect$county_fips,sum))
total_64 <- as.vector(by(elect$johnson,elect$county_fips,sum))
total_72 <- as.vector(by(elect$total_72,elect$county_fips,sum))
elect <- NULL
elect <- data.frame(cbind(county,johnson,mcgovern,total_64,total_72))
elect <- elect[elect$total_64!=9999999 & elect$total_72!=9999999,]
length(levels(factor(saiz$msanecma)))
elect$county[elect$county==12025] <- 12086
saiz <- merge(saiz,elect,by.x="county",by.y="county")
length(levels(factor(saiz$msanecma)))
summary(saiz)

# land area
gaz <- read.table("Data/2014_Gaz_counties_national.txt",sep="\t",header=T) # tweaked: O'Brien county
#STUSPS	GEOID	ANSICODE	NAME	ALAND	AWATER	ALAND_SQMI	AWATER_SQMI	INTPTLAT	INTPTLONG
#AL	01001	00161526	Autauga County	1539582278	25775735	594.436	9.952	32.536382	-86.644490
goodnames <- c("GEOID","ALAND_SQMI")
for (n in names(gaz)) if (n %in% goodnames==FALSE) gaz[[n]] <- NULL
saiz <- merge(saiz,gaz,by.x="county",by.y="GEOID")
length(levels(factor(saiz$msanecma)))


church <- read.csv("Data/church_71.csv")
names(church)
church$non_fundie_total <- (church$CATHOL_A+church$AMBAPT_A+church$PRESUS_A + church$FREEME_A + church$LCA_A + church$EPISC_A + church$UCC_A)
church$total <- church$TOTADH
church <- church[!is.na(church$non_fundie_total) & !is.na(church$total),]
summary(church)
for (n in names(church)) {
	if (n %in% list("non_fundie_total","FIPS","total")==FALSE) {
		church[[n]] <- NULL
	}
}
church$FIPS[church$FIPS==12025] <- 12086

jew <- read.csv("Data/puma_jew_brandeis.csv")
puma <- read.csv("Data/county_puma.csv",skip=1)
puma$puma <- puma$puma12+puma$FIPS.state*100000
goodnames <- c("county","puma","county.to.puma2k.alloc.factor")
for (n in names(puma)) {
	if (n %in% goodnames==FALSE) {
		puma[[n]] <- NULL
	}
}
summary(jew)
summary(puma)
jew <- merge(jew,puma,by="puma")
jew <- jew[jew$county.to.puma2k.alloc.factor==ave(jew$county.to.puma2k.alloc.factor,jew$county,FUN=max),]
jew$county.to.puma2k.alloc.factor <- NULL
jew[jew$county==12011,]
summary(church)
church <- merge(church,jew,by.x="FIPS",by.y="county",all.x=TRUE)
summary(church)

saiz <- merge(saiz,church,by.x="county",by.y="FIPS")
length(levels(factor(saiz$msanecma)))
length(levels(factor(saiz$msanecma[!is.na(saiz$jew)])))

cbsa <- read.csv("Data/cbsa_13.csv",skip=2,header=TRUE)
cbsa$rr <- runif(length(cbsa$CBSA.Code))
cbsa <- cbsa[cbsa$rr==ave(cbsa$rr,cbsa$CBSA.Code,FUN=max),]
cbsa <- data.frame(cbsa$CBSA.Code,cbsa$CBSA.Title)
names(cbsa) <- c("cbsa","cbsa_name")


fm <- read.table("Data/fmhpi_reshape.csv",sep=";",header=F)
#Abilene, TX;1975;01;43.93
names(fm) <- c("cbsa_name","year","month","fmhpi")
fm <- read.table("Data/fmhpi_reshape.csv",sep=";",header=F)
#Abilene, TX;1975;01;43.93
names(fm) <- c("cbsa_name","year","month","fmhpi")

cpi <- read.csv("Data/cpi_monthly.csv",skip=10)
cpi$year <- as.numeric(substring(cpi$observation_date,1,4))
cpi$month <- as.numeric(substring(cpi$observation_date,6,7))
cpi$observation_date <- NULL
summary(cpi)
summary(fm)
fm <- merge(fm,cpi,by=c("year","month"))
fm$fmhpi <- fm$fmhpi/fm$CPIAUCNS
fm <- merge(fm,cbsa,by="cbsa_name")

long <- 0

if (long==1) {
	fm$max_cycle <- MISSING
	for (date in seq(1976,2014)) {
		fm[[paste("is",date)]] <- ave(fm$fmhpi*(fm$year==date & fm$month==1),fm$cbsa,FUN=max)
	}
	for (start in seq(1976,2012)) {
		print(paste("start",start))
		for (peak in seq((start+1),2013)) {
			print(paste("peak",peak))
			for (stop in seq((peak+1),2014)) {
				print(paste("stop",stop))
				fm$max_cycle <- pmax(fm$max_cycle, (fm[[paste("is",peak)]]^2/(fm[[paste("is",start)]]*fm[[paste("is",stop)]]))/(stop-start))
			}
		}
	}
}
if (long==0) {
	write.csv(fm,"docs/supply_instrument/data/for_julia.csv",row.names=FALSE)
	system("julia docs/supply_instrument/scripts/max_cycle.jl > docs/supply_instrument/data/max_cycle.csv") # already done
}

gm <- fm[fm$month==1,]
for (j in seq(1977,2014)) {
	gm$eps <- log(ave(gm$fmhpi*(gm$year==j),gm$cbsa,FUN=max)/ave(gm$fmhpi*(gm$year==(j-1)),gm$cbsa,FUN=max))
	gm$grow_fm[gm$year==j] <- gm$eps[gm$year==j]
}
gm <- gm[gm$year>1976,]
gm$vol <- 100*ave(gm$grow_fm,gm$cbsa,FUN=sd)
gm <- gm[gm$year==1977,]
gn <- c("cbsa","vol")
for (n in names(gm)) {
	if (n %in% gn==FALSE) {
		gm[[n]] <- NULL
	}
}
gm


mc <- read.table("docs/supply_instrument/data/max_cycle.csv",header=FALSE,sep=";")
names(mc) <- c("cbsa_name","max_cycle","start","peak","stop")
fm <- merge(mc,fm,by="cbsa_name")

max_cycle <- as.vector(by(fm$max_cycle,fm$cbsa,max))
grower <- log(as.vector(by(fm$fmhpi*(fm$year==2014 & fm$month==1),fm$cbsa,max)/by(fm$fmhpi*(fm$year==1980 & fm$month==1),fm$cbsa,max)))
peak <- as.vector(by(fm$fmhpi*(fm$year==2007 & fm$month==1),fm$cbsa,max))
starter <- as.vector(by(fm$fmhpi*(fm$year==2000 & fm$month==1),fm$cbsa,max))
ender <- as.vector(by(fm$fmhpi*(fm$year==2012 & fm$month==1),fm$cbsa,max))
cbsa <- as.vector(by(fm$cbsa,fm$cbsa,mean))
fmhpi <- data.frame(cbsa,grower,log(peak/starter)/7 - log(ender/peak)/5,max_cycle)
levels(factor(fmhpi$cbsa))
levels(factor(saiz$cbsa))
names(fmhpi) <- c("cbsa","grow_fmhpi","boom_2000s","max_cycle")
saiz <- merge(saiz,fmhpi,by="cbsa")
length(levels(factor(saiz$msanecma)))


# immigrant 1980
immigrant <- read.csv("docs/supply_instrument/data/conspuma_immigrant.csv",sep="\t",na.strings=c("","."))
high_educ <- read.csv("docs/supply_instrument/data/cons_puma_high_educ.txt",sep="\t",na.strings=c("","."))
college <- read.csv("docs/supply_instrument/data/educ_ipums.csv") # 4 years college +
bartik <- read.csv("docs/supply_instrument/data/bartik_80_11.csv",na.strings=c("",".")) # bartik_alternative no mechanical bad
rent <- read.csv("docs/supply_instrument/data/conspuma_rent_80_11_3.csv",na.strings=c("",".")) # bartik_alternative no mechanical bad
names(high_educ) <- c("conspuma","high_educ")
names(bartik) <- c("conspuma","bartik")
immigrant <- merge(immigrant,high_educ,by="conspuma")
immigrant <- merge(immigrant,bartik,by="conspuma")
immigrant <- merge(immigrant,college,by="conspuma")
immigrant <- merge(immigrant,rent,by="conspuma")
summary(immigrant)
map_1 <- read.csv("docs/supply_instrument/data/conspuma_components.csv",na.strings=c(",",""))
summary(map_1)
immigrant <- merge(immigrant,map_1,by="conspuma")
map_2 <- read.csv("docs/supply_instrument/data/county_group_ipums_80.csv",na.strings=c(".",""))
summary(map_2)
map_2
map_2$state_cntygp98 <- 1000*map_2$State.FIPS.Code + map_2$X1980.County.Group
map_2$county <- 1000*map_2$State.FIPS.Code + map_2$County.FIPS.code
map_2$county[map_2$county==12025] <- 12086
map_2$State.FIPS.Code <- map_2$County.FIPS.code <- NULL
immigrant <- merge(immigrant,map_2,by="state_cntygp98")
summary(immigrant)
high_educ <- as.vector(by(immigrant$high_educ,immigrant$county,mean))
bartik <- as.vector(by(immigrant$bartik,immigrant$county,mean))
county <- as.vector(by(immigrant$county,immigrant$county,mean))
college_plus_80 <- as.vector(by(immigrant$educ_10_11_80,immigrant$county,mean))
college_plus_11_3 <- as.vector(by(immigrant$educ_10_11_11_3,immigrant$county,mean))
rent_80 <- as.vector(by(immigrant$Rent_1980,immigrant$county,mean))
rent_11_3 <- as.vector(by(immigrant$Rent_11_3,immigrant$county,mean))
immigrant <- as.vector(by(immigrant$immigrant,immigrant$county,mean))
immigrant <- data.frame(cbind(immigrant,high_educ,bartik,county,college_plus_80,college_plus_11_3,rent_80,rent_11_3))
summary(immigrant)
length(levels(factor(saiz$msanecma)))
saiz <- merge(saiz,immigrant,by="county")
length(levels(factor(saiz$msanecma)))
length(saiz$county)


saiz$weight <- saiz$afact*saiz$units_80/ave(saiz$afact*saiz$units_80,saiz$cbsa,FUN=sum)
summary(saiz)
for (v in list("high_educ","college_plus_11_3","college_plus_80","bartik","grow_fmhpi","max_cycle","WRLURI","elasticity","sand","coastal","unaval","cbsa","ALAND_SQMI","immigrant","jew","population","rent_80","rent_11_3")) {
	assign(v,as.vector(by(saiz$weight*saiz[[v]],saiz$cbsa,FUN=sum,na.rm=TRUE))) # for jews
}
for (v in list("total","non_fundie_total","units_80","units_12","johnson","mcgovern","total_64","total_72")) {
	assign(v,as.vector(by(saiz[[v]],saiz$cbsa,FUN=sum)))
}
df <- data.frame(cbind(high_educ,college_plus_11_3,college_plus_80,bartik,units_80,units_12,grow_fmhpi,max_cycle,johnson,mcgovern,total_64,total_72,WRLURI,elasticity,sand,coastal,total,non_fundie_total,unaval,cbsa,ALAND_SQMI,immigrant,jew,rent_80,rent_11_3))

#incraesed sorting
df$coastal <- df$coastal>0
sd(df$high_educ)/mean(df$high_educ)
sd(df$college_plus_80)/mean(df$college_plus_80)
sd(df$college_plus_11_3)/mean(df$college_plus_11_3)
cbind(by(df$college_plus_11_3,df$coastal,mean),by(df$college_plus_80,df$coastal,mean),by(df$high_educ,df$coastal,mean))
round(cor(cbind(df$college_plus_80,df$high_educ,df$college_plus_11_3/df$college_plus_80,df$college_plus_11_3,df$bartik,df$elasticity,df$coastal,df$sand,df$density)),2)

pdf("docs/supply_instrument/text/clusters.pdf")
z <- lm(df$grow_fmhpi ~ df$jew)$residuals
plot(df$high_educ,z)
plot(df$high_educ,df$grow_fmhpi)
z <- lm(df$grow_fmhpi ~ df$elasticity)$residuals
plot(df$elasticity,z)
plot(df$elasticity,df$grow_fmhpi)
dev.off()
z <- lm(df$grow_fmhpi ~ log(df$elasticity))$residuals
plot(log(df$elasticity),z)
plot(log(df$elasticity),df$grow_fmhpi)
dev.off()

df <- merge(df,gm,by="cbsa")
cor(df$vol,df$max_cycle)
df$grow_rent <- log(df$rent_11_3)-log(df$rent_80)
df$sand <- df$sand>0
df$coastal <- df$coastal>0
df$protestant <- 1-df$non_fundie_total/df$total
df$density <- (df$units_80/1000)/df$ALAND_SQMI
df$grow_units <- log(df$units_12/df$units_80)
df$dem_share <- df$mcgovern/df$total_72 # too many mv ow
round(cor(cbind(df$unaval,df$WRLURI,df$protestant,df$elasticity,df$sand,df$coastal,df$high_educ,df$bartik,df$immigrant)),2)
summary(df)
jf <- df[!is.na(df$jew) & df$jew>0,]
summary(jf)
sd(jf$jew)
cbind(jf$protestant,jf$jew,jf$cbsa)
round(cor(cbind(jf$unaval,jf$WRLURI,jf$protestant,jf$jew,jf$elasticity,jf$sand,jf$coastal,jf$high_educ,jf$bartik,jf$immigrant)),2)
by(df$units_80,is.na(df$jew),mean)

summary(lm(df$grow_fmhpi ~ df$coastal))
summary(lm(df$grow_fmhpi ~ df$elasticity))
summary(lm(df$grow_fmhpi ~ df$elasticity+df$coastal))
summary(lm(df$grow_fmhpi ~ df$elasticity+df$bartik+df$high_educ+df$immigrant+df$coastal))
summary(lm(df$grow_units ~ df$bartik+df$high_educ+df$immigrant+df$coastal))
summary(lm(df$grow_units ~ df$elasticity+df$bartik+df$high_educ+df$immigrant+df$coastal))

df$bartik_elasticity <- df$bartik*df$elasticity
df$bartik_bartik <- df$bartik*df$bartik
df$high_educ_high_educ <- df$high_educ*df$high_educ
df$high_educ_bartik <- df$high_educ*df$bartik
df$high_educ_elasticity <- df$high_educ*df$elasticity
df$elasticity_elasticity <- df$elasticity*df$elasticity
df$high_educ_immigrant <- df$high_educ*df$immigrant
df$bartik_immigrant <- df$immigrant*df$bartik
df$elasticity_immigrant <- df$immigrant*df$elasticity
df$immigrant_immigrant <- df$immigrant^2


for (suspect in list("unaval","WRLURI","elasticity")) {
	print(suspect)
	outreg(list(
	    reg(df,"grow_fmhpi",list(suspect)),
	    reg(df,"grow_fmhpi",list(suspect,"high_educ","immigrant","bartik")),
	    reg(df,"grow_fmhpi",list(suspect,"high_educ","immigrant","bartik","density","coastal","sand"))
	))
	outreg(list(
    	    reg(df,"grow_units",list(suspect)),
	    reg(df,"grow_units",list(suspect,"high_educ","immigrant","bartik")),
	    reg(df,"grow_units",list(suspect,"high_educ","immigrant","bartik","density","coastal","sand"))
	))
	outreg(list(
    	    reg(df,"vol",list(suspect)),
	    reg(df,"vol",list(suspect,"high_educ","immigrant","bartik")),
	    reg(df,"vol",list(suspect,"high_educ","immigrant","bartik","density","coastal","sand"))
    ))
}

for (suspect in list("unaval","WRLURI","elasticity","protestant")) {
outreg(list(
    reg(jf,"grow_fmhpi",list(suspect)),
    reg(jf,"grow_fmhpi",list(suspect,"high_educ","immigrant","bartik","jew")),
    reg(jf,"grow_fmhpi",list(suspect,"high_educ","immigrant","bartik","density","coastal","sand","jew"))
    ))
outreg(list(
    reg(jf,"grow_units",list(suspect)),
    reg(jf,"grow_units",list(suspect,"high_educ","immigrant","bartik","jew")),
    reg(jf,"grow_units",list(suspect,"high_educ","immigrant","bartik","density","coastal","sand","jew"))
))
outreg(list(
    reg(jf,"vol",list(suspect)),
    reg(jf,"vol",list(suspect,"high_educ","immigrant","bartik","jew")),
    reg(jf,"vol",list(suspect,"high_educ","immigrant","bartik","density","coastal","sand","jew"))
))
}

outreg(list(
	    reg(df,"grow_fmhpi","coastal"),
	    reg(df,"grow_units","coastal"))
)

outreg(list(
	    reg(df,"grow_fmhpi",list("elasticity","bartik","bartik_elasticity")),
	    reg(df,"grow_fmhpi",list("elasticity","bartik","bartik_elasticity","bartik_bartik","high_educ","high_educ_high_educ","high_educ_bartik","immigrant","bartik_immigrant","high_educ_immigrant","immigrant_immigrant")),
	    reg(df,"grow_fmhpi",list("elasticity","bartik","bartik_elasticity","bartik_bartik","high_educ","high_educ_high_educ","high_educ_bartik","high_educ_elasticity","immigrant","bartik_immigrant","high_educ_immigrant","immigrant_immigrant", "elasticity_immigrant")),
	    reg(df,"grow_units",list("elasticity","bartik","bartik_elasticity")),
	    reg(df,"grow_units",list("elasticity","bartik","bartik_elasticity","bartik_bartik","high_educ","high_educ_high_educ","high_educ_bartik","immigrant","bartik_immigrant","high_educ_immigrant","immigrant_immigrant")),
	    reg(df,"grow_units",list("elasticity","bartik","bartik_elasticity","bartik_bartik","high_educ","high_educ_high_educ","high_educ_bartik","high_educ_elasticity","immigrant","bartik_immigrant","high_educ_immigrant","immigrant_immigrant","elasticity_immigrant"))
))

# show iv ng
iv <- function(y,x,z,fe_var="") {
	if (fe_var!="") {
		y[fe_var!=min(fe_var)] <- y[fe_var!=min(fe_var)] - ave(y[fe_var!=min(fe_var)],fe_var[fe_var!=min(fe_var)])
		for (i in seq(ncol(x))) x[fe_var!=min(fe_var),i] <- x[fe_var!=min(fe_var),i] - ave(x[fe_var!=min(fe_var),i],fe_var[fe_var!=min(fe_var)]) # omit last category of fe_var
		for (i in seq(ncol(z))) z[fe_var!=min(fe_var),i] <- z[fe_var!=min(fe_var),i] - ave(z[fe_var!=min(fe_var),i],fe_var[fe_var!=min(fe_var)]) # omit last category of fe_var
	}
	pz <- z%*%solve(t(z)%*%z)%*%t(z)
	xhat <- pz%*%x
	beta <- solve(t(xhat)%*%xhat)%*%t(xhat)%*%y
	resid <- y-x%*%beta
	mid_mat <- 0*t(z)%*%z
	for (i in seq(length(y))) {
		mid_mat <- mid_mat + z[i,]%*%t(z[i,])*resid[i]*resid[i]
	}
	var_mat <- solve(t(x)%*%pz%*%x)%*%t(x)%*%z%*%solve(t(z)%*%z)%*%mid_mat%*%solve(t(z)%*%z)%*%t(z)%*%x%*%solve(t(x)%*%pz%*%x) # from Cameron lecture notes Davis
	print("BETA")
	print(beta)
	print("VAR_MAT")
	print(sqrt(diag(var_mat)))
	print("CHECKER")
	print(summary(xhat))
	print(summary(x))
}
y <- df$grow_units
x <- cbind(rep(1,length(df$elasticity)),df$grow_rent,df$high_educ,df$immigrant,df$bartik,df$density)
z <- cbind(rep(1,length(df$elasticity)),df$elasticity,df$high_educ,df$immigrant,df$bartik,df$density)
summary(lm(y~x))
iv(y,x,z)
x <- cbind(rep(1,length(df$elasticity)),df$grow_rent)
z <- x # Just to confirm
summary(lm(y~x))
iv(y,x,z)
y <- df$grow_units
x <- cbind(rep(1,length(df$elasticity)),df$grow_rent,df$bartik,df$elasticity,df$bartik_bartik,df$high_educ,df$high_educ_high_educ,df$high_educ_bartik,df$immigrant,df$bartik_immigrant,df$high_educ_immigrant,df$immigrant_immigrant)
z <- cbind(rep(1,length(df$elasticity)),df$elasticity,df$bartik,df$bartik_elasticity,df$bartik_bartik,df$high_educ,df$high_educ_high_educ,df$high_educ_bartik,df$high_educ_elasticity,df$immigrant,df$bartik_immigrant,df$high_educ_immigrant,df$immigrant_immigrant,df$elasticity_immigrant)
summary(lm(y~x))
iv(y,x,z)

# Now the fmhpi high frequency panel
fm <- read.table("Data/fmhpi_reshape.csv",sep=";",header=F)
#Abilene, TX;1975;01;43.93
names(fm) <- c("cbsa_name","year","month","fmhpi")

cpi <- cpi[order(cpi$year+(cpi$month-1)/12),]
cpi$inflation <- c(rep(NA,12),diff(log(cpi$CPIAUCNS),lag=12))
fm <- merge(fm,cpi,by=c("year","month"))

cbsa = read.csv("Data/cbsa_13.csv",skip=2,header=TRUE)
cbsa$rr <- runif(length(cbsa$CBSA.Code))
cbsa <- cbsa[cbsa$rr==ave(cbsa$rr,cbsa$CBSA.Code,FUN=max),]
cbsa <- data.frame(cbsa$CBSA.Code,cbsa$CBSA.Title)
names(cbsa) <- c("cbsa","cbsa_name")
fm <- merge(fm,cbsa,by="cbsa_name")
summary(fm)
fm <- merge(fm,df,by="cbsa")
summary(fm)
# 30-year interest rates by month and year
m <- read.table("Data/MORTG.txt",skip=14)
summary(m)
m$year <- as.numeric(substring(m$V1,1,4))
m$month <- as.numeric(substring(m$V1,6,7))
summary(m)
m$V1 <- NULL
m$mtg <- as.numeric(substring(m$V2,1,6))
m$V2 <- NULL
fm <- merge(fm,m,by=c("year","month"))

fm$mtg <- fm$mtg-fm$inflation
fm$date  <- fm$year+fm$month/12
cor(fm$date,fm$mtg)
fm$emtg <- fm$elasticity*fm$mtg
fm$lfmhpi <- log(fm$fmhpi)
fm$edate <- fm$elasticity*fm$date
fm$bartik_mtg <- fm$bartik*fm$mtg
fm$high_educ_mtg <- fm$high_educ*fm$mtg
fm$immigrant_mtg <- fm$immigrant*fm$mtg
fm$coastal_mtg <- fm$coastal*fm$mtg
fm <- fm[fm$date>1993 & fm$date<2008,]
pdf("docs/supply_instrument/text/politics.pdf")
plot(df$dem_share,exp(df$grow_fmhpi))
dev.off()
cbind(df$mcgovern,df$johnson,df$total_64,df$total_72,df$cbsa)

saiz <- saiz[saiz$population>500000,]
saiz <- saiz[order(-saiz$elasticity),]
z <- unique(cbind(substring(saiz$msaname,1,35),round(log(ave(saiz$units_12,saiz$msaname,FUN=sum)/ave(saiz$units_80,saiz$msaname,FUN=sum)),2),round(ave(saiz$grow_fmhpi,saiz$msaname,FUN=mean),2)))
saiz <- saiz[order(saiz$elasticity),]
y <- unique(cbind(substring(saiz$msaname,1,35),round(log(ave(saiz$units_12,saiz$msaname,FUN=sum)/ave(saiz$units_80,saiz$msaname,FUN=sum)),2),round(ave(saiz$grow_fmhpi,saiz$msaname,FUN=mean),2)))
cbind(z[1:10,],y[1:10,])
pdf("docs/supply_instrument/text/empirical_demand_supply.pdf")
plot(df$grow_units,df$grow_fmhpi,cex=1/df$elasticity,xlab="Quantity growth 1980-2012",ylab="Real price growth 1980-2014")
dev.off()
print(sumstats(df,names(df)))

summary(df$grow_units>0)
outreg(list(
	    reg(fm,"lfmhpi",list("date","emtg"),cluster_var="cbsa",fe_var="cbsa",dummy_list=list("date")),
	    reg(fm,"lfmhpi",list("date","emtg","edate"),cluster_var="cbsa",fe_var="cbsa",dummy_list=list("date")),
	    reg(fm,"lfmhpi",list("date","emtg","edate","high_educ_mtg","bartik_mtg","immigrant_mtg"),cluster_var="cbsa",fe_var="cbsa",dummy_list=list("date")))
,silentvars=paste("date",levels(factor((fm$date)))))


q("no")

